unit FavEditor;

interface

uses
  FavMenuData,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, ActnList;

type
  TFavEditor = class(TForm)
    pnlClient: TPanel;
    pnlButtonsRight: TPanel;
    pnlSep1: TPanel;
    FileList: TListBox;
    pnlBottom: TPanel;
    pnlButtonsDlg: TPanel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    ActionList1: TActionList;
    actUp: TAction;
    actDown: TAction;
    actAdd: TAction;
    actSep: TAction;
    actDelete: TAction;
    OpenDialog: TOpenDialog;
    cbxDisplayMode: TComboBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure actSepExecute(Sender: TObject);
    procedure actAddExecute(Sender: TObject);
    procedure actDeleteExecute(Sender: TObject);
    procedure ActionList1Update(Action: TBasicAction;
      var Handled: Boolean);
    procedure actUpExecute(Sender: TObject);
    procedure actDownExecute(Sender: TObject);
    procedure FileListDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    procedure LoadFromData(data: TFavData);
    procedure SaveToData(data: TFavData);
  public
    function Edit(data:TFavData):boolean;
  end;

implementation

{$R *.DFM}

uses
  registry, ToolsAPI;

resourcestring
  sSeperator = '- seperator -';
    
{ TFavEditor }

procedure TFavEditor.LoadFromData(data:TFavData);
 var i:integer; fn:string;
begin
  FileList.clear;
  for i := 0 to data.count-1 do
  begin
    fn := data[i];
    Filelist.items.addobject(fn, TObject(FileExists(fn)));
  end;

  cbxDisplayMode.itemindex := ord(data.DisplayMode);
end;

procedure TFavEditor.SaveToData(data:TFavData);
 var i:integer; 
begin
  data.clear;
  for i := 0 to FileList.items.count-1 do
  begin
    data.add(Filelist.items[i] );
  end;

  data.DisplayMode := TDisplayMode( cbxDisplayMode.itemindex );
end;

function TFavEditor.Edit(data: TFavData): boolean;
begin
  LoadFromData(data);

  result := (ShowModal = mrOK);

  if result
    then SaveToData(data);
end;

procedure TFavEditor.FormCreate(Sender: TObject);
 var r:TRegistry; key:string; pos:TRect;
begin
  key := (BorlandIDEServices as IOTAServices).GetBaseRegistryKey + sRegSubKey;

  r := TRegistry.create;
  try
    r.RootKey := HKEY_CURRENT_USER;
    if r.OpenKeyReadOnly(key) then
      if r.ValueExists('EditorPos') then
      begin
        r.ReadBinaryData('EditorPos',pos, sizeof(pos) );
        BoundsRect := pos;
      end;

  finally
    r.free;
  end;
end;

procedure TFavEditor.FormDestroy(Sender: TObject);
 var r:TRegistry; key:string; pos:TRect;
begin
  key := (BorlandIDEServices as IOTAServices).GetBaseRegistryKey + sRegSubKey;

  r := TRegistry.create;
  try
    r.RootKey := HKEY_CURRENT_USER;
    if r.OpenKey(key, true) then
    begin
      pos := BoundsRect;
      r.WriteBinaryData('EditorPos',pos, sizeof(pos) );
    end;

  finally
    r.free;
  end;
end;

procedure TFavEditor.actSepExecute(Sender: TObject);
 var i:Integer;
begin
  i := FileList.ItemIndex;
  if (i<0) then i := FileList.items.count-1;

  FileList.items.insertObject(i+1, sSeperator, TObject(false) );
  FileList.ItemIndex := i+1;
end;

procedure TFavEditor.actAddExecute(Sender: TObject);
 var i:Integer;
begin
  if OpenDialog.execute then
  begin
    i := FileList.ItemIndex;
    if (i<0) then i := FileList.items.count-1;

    FileList.items.insertObject(i+1, OpenDialog.FileName, TObject(FileExists(OpenDialog.FileName)) );
    FileList.ItemIndex := i+1;
  end;
end;

procedure TFavEditor.actDeleteExecute(Sender: TObject);
 var i:integer;
begin
  i := FileList.itemindex;
  FileList.items.delete (i);
  if (i<FileList.items.count)
    then FileList.itemindex := i
    else FileList.itemindex := FileList.items.count-1;
end;

procedure TFavEditor.ActionList1Update(Action: TBasicAction;  var Handled: Boolean);
begin
  actDelete.Enabled := FileList.itemindex>=0;
  actUp.Enabled     := FileList.itemindex>0;
  actDown.enabled   := (FileList.itemindex < (FileList.items.count-1))
                       and (FileList.itemindex>=0);

  Handled := true;
end;

procedure TFavEditor.actUpExecute(Sender: TObject);
 var i:integer;
begin
  i := FileList.itemindex;
  FileList.items.Exchange(i, i-1);
  FileList.itemindex := i-1;
end;

procedure TFavEditor.actDownExecute(Sender: TObject);
 var i:integer;
begin
  i := FileList.itemindex;
  FileList.items.Exchange(i, i+1);
  FileList.itemindex := i+1;
end;

type TLBHack = class(TCustomListBox);
procedure TFavEditor.FileListDrawItem(Control: TWinControl; Index: Integer;  Rect: TRect; State: TOwnerDrawState);
 var saved:TDrawItemEvent;
begin
  if FileList.items.objects[index] = TObject(false) then
    FileList.Canvas.Font.color := clGrayText;

  saved := FileList.OnDrawItem;
  FileList.OnDrawItem := nil;
  try

    TLBHack(FileList).DrawItem(Index, Rect, State);

  finally
    FileList.OnDrawItem := saved;
  end;
end;

end.
