unit FavMenuData;

interface

uses
  toolsapi, classes;

type
  TDisplayMode =(dmFull, dmFileName, dmRelative);

  TFavData = class(TStringList)
  private
    fDisplayMode:TDisplayMode;
  private
    function  GetDisplayValue(index: integer): string;
    procedure SetDisplayMode(const Value: TDisplayMode);
  public
    procedure Load;
    procedure Save;
    function  Add(val:string):boolean; reintroduce;
  public
    property DisplayMode:TDisplayMode read fDisplayMode write SetDisplayMode;
    property DisplayValues[index:integer]:string read GetDisplayValue;
    property Values[index:integer]:string read Get;
  public
    constructor Create;
  end;

function  CurrentFileName:string;
function  GetProjectGroup: IOTAProjectGroup;
function  CurrentProjectFilename:string;

resourcestring
  sRegSubKey         = '\Favourites';

implementation

uses
  sysutils, windows, registry;


function CurrentFileName: string;
 var m:IOTAModule;
begin
  m := (BorlandIDEServices as IOTAModuleServices).CurrentModule;
  if assigned(m)
    then result := m.FileName
    else result := '';
end;

function CurrentProjectFilename: string;
var
  ProjectGroup: IOTAProjectGroup; m:IOTAModule;
begin
  ProjectGroup := GetProjectGroup;

  if Assigned(ProjectGroup) then
    m := ProjectGroup.ActiveProject
  else
    m := nil;

  if assigned(m)
    then result := m.FileName
    else result := '';
end;

function GetProjectGroup: IOTAProjectGroup;
var
  IModuleServices: IOTAModuleServices;
  I: Integer;
begin
  IModuleServices := BorlandIDEServices as IOTAModuleServices;
  for I := 0 to IModuleServices.ModuleCount - 1 do
    if IModuleServices.Modules[I].QueryInterface(IOTAProjectGroup, Result) = S_OK then
      Exit;
  Result := nil;
end;

{ TFavData }

function TFavData.Add(val: string): boolean;
begin
  if IndexOf(val)>= 0
    then result := false
    else result := (inherited Add(val)>=0);
end;

constructor TFavData.Create;
begin
  inherited;
  Load;
end;

function TFavData.GetDisplayValue(index: integer): string;
 var dir:string;
begin
  result := strings[index];
  dir := UpperCase(ExtractFilePath(CurrentProjectFileName));

  if  length(result)<>0 then
  begin
    if result[1] = '-' then
      result := '-'
    else
      case DisplayMode of
        dmRelative :
          if dir = uppercase(copy(result,1,length(dir)) )
            then system.delete(result, 1, length(dir) );
        dmFilename :
          result := ExtractFileName(result);
        dmFull:
          {};
       end {case};

  end;
end;

procedure TFavData.Load;
 var r:TRegistry; key:string; i,count:integer; val:string;
begin
  key := (BorlandIDEServices as IOTAServices).GetBaseRegistryKey + sRegSubKey;

  r := TRegistry.create;
  try
    BeginUpdate;
    try

      r.RootKey := HKEY_CURRENT_USER;
      if not r.OpenKeyReadOnly(key)
        then exit;
      count := r.ReadInteger('count');
      fDisplayMode := TDisplayMode( r.ReadInteger('DisplayMode') );

      for i := 0 to count-1 do
      begin
        val := r.ReadString( inttostr(i) );
        if (val<>'')
          then Add(val);
      end;

    finally
      EndUpdate;
    end;

  finally
    r.free;
  end;
end;

procedure TFavData.Save;
 var r:TRegistry; key:string; i:Integer; 
begin
  key := (BorlandIDEServices as IOTAServices).GetBaseRegistryKey + sRegSubKey;

  r := TRegistry.create;
  try
    r.RootKey := HKEY_CURRENT_USER;
    if not r.OpenKey(key, true)
      then exit;
    r.WriteInteger('count', count);
    r.WriteInteger('DisplayMode', Ord(fDisplayMode) );

    for i := 0 to count-1 do
    begin
      r.WriteString(inttostr(i), strings[i] );
    end;

  finally
    r.free;
  end;
end;

procedure TFavData.SetDisplayMode(const Value: TDisplayMode);
begin
   if (value<>fDisplayMode) then
   begin
     Changing;
     fDisplayMode := value;
     Changed;
   end;
end;

end.
